<?php
/**
 * @version   $Id: jomsocialkunenasync.php,v 9dcce53693a7 2011/02/03 18:06:24 likemandrake $
 * @author    Piotr Minkina <likemandrake@o2.pl>
 * @copyright Copyright (C) 2011 Piotr Minkina
 * @license   GNU/GPL version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

class plgSystemJomSocialKunenaSync extends JPlugin
{
    protected $_sync;
    protected $_userdata;
    protected $_dbmap;
    protected $_map;

    public function __construct(&$subject, $config = array())
    {
        parent::__construct($subject, $config);
        $this->_type = 'community';
        $this->_sync = true;
    }

    public function onAfterStoreUser($user, $isnew, $success, $msg)
    {
        if ($success && true == $this->_sync)
        {
            $this->_initialize();
            $this->_userdata = $user;
            $mapped =& $this->_mapValues(false);
            $this->_sync = false;
            $this->_kunenaToJomSocial($user['id'], $mapped);
        }
    }

    public function onBeforeProfileUpdate($userid, $values)
    {
        $this->_userdata = $values;
        return true;
    }

    public function onAfterProfileUpdate($userid, $success)
    {
        if ($success && true == $this->_sync)
        {
            $this->_initialize();
            $mapped =& $this->_mapValues(true);
            $this->_sync = false;
            $this->_jomSocialToKunena($userid, $mapped);
        }
    }

    protected function _initialize()
    {
        $this->_dbmap = array
        (
            'personaltext'  => 'personalText',
            'birthdate'     => 'birthdate',
            'location'      => 'location',
            'gender'        => 'gender',
            'websiteurl'    => 'websiteurl',
            'twitter'       => 'TWITTER',
            'facebook'      => 'FACEBOOK',
            'myspace'       => 'MYSPACE',
            'skype'         => 'SKYPE',
            'linkedin'      => 'LINKEDIN',
            'delicious'     => 'DELICIOUS',
            'friendfeed'    => 'FRIENDFEED',
            'digg'          => 'DIGG',
            'yim'           => 'YIM',
            'aim'           => 'AIM',
            'gtalk'         => 'GTALK',
            'icq'           => 'ICQ',
            'msn'           => 'MSN',
            'blogspot'      => 'BLOGSPOT',
            'flickr'        => 'FLICKR',
            'bebo'          => 'BEBO',
            'signature'     => 'signature',
            'messageordering' => 'ordering',
            'hidemail'      => 'hideEmail',
            'showonline'    => 'showOnline'
        );
        $this->_map   = array();

        foreach (array_keys($this->_dbmap) as $param)
        {
            $jomsocial = $this->params->get($param, 0);
            if (!empty($jomsocial))
            {
                $this->_map[$param] = $jomsocial;
            }
        }
    }

    protected function & _mapValues($toKunena = true)
    {
        $mapped = array();

        foreach ($this->_map as $kunena => $jomsocial)
        {
            if ($toKunena)
            {
                $from = $jomsocial;
                $to   = $this->_dbmap[$kunena];
            }
            else
            {
                $from = $kunena;
                $to   = $jomsocial;
            }

            $method = '_' . strtolower($kunena) . 'Mapper';
            if (method_exists($this, $method))
            {
                $this->$method($mapped, $from, $to, $toKunena);
            }
            else
            {
                $this->_defaultMapper($mapped, $from, $to);
            }
        }
        return $mapped;
    }

    protected function _kunenaToJomSocial($userid, &$values)
    {
        require_once JPATH_ROOT . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'core.php';
        $model = CFactory::getModel('profile');

        foreach ($values as $fieldCode => $value)
        {
            $model->updateUserData($fieldCode, $userid, $value);
        }
    }

    protected function _jomSocialToKunena($userid, &$values)
    {
        require_once JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_kunena' . DS . 'api.php';
        $user = KunenaFactory::getUser($userid);

        $user->setProperties($values);
        $user->save();
    }

    protected function _mapArrayHelper(&$map, $from, $toKunena)
    {
        $to = $this->_userdata[$from];
        if ($toKunena)
        {
            $map = array_flip($map);
        }
        if (!isset($map[$to]))
        {
            reset($map);
            return current($map);
        }
        return $map[$to];
    }

    protected function _defaultMapper(&$mapped, $from, $to)
    {
        $mapped[$to] = (string) $this->_userdata[$from];
    }

    protected function _genderMapper(&$mapped, $from, $to, $toKunena)
    {
        $male   = $this->params->get('map_gender_male', 'Male');
        $female = $this->params->get('map_gender_female', 'Female');

        $map    = array(0 => '', 1 => $male, 2 => $female);

        $mapped[$to] = $this->_mapArrayHelper($map, $from, $toKunena);
    }

    protected function _birthdateMapper(&$mapped, $from, $to, $toKunena)
    {
        if ($toKunena)
        {
            $date = explode(' ', $this->_userdata[$from]);
            $date = array_shift($date);

            $mapped[$to] = $date;
        }
        else
        {
            $mapped[$to] = $this->_userdata[$from . '1']
                         . '-' . $this->_userdata[$from . '2']
                         . '-' . $this->_userdata[$from . '3']
                         . ' 23:59:59';
        }
    }

    protected function _websiteurlMapper(&$mapped, $from, $to, $toKunena)
    {
        $websitename = $this->params->get('map_websitename', 0);

        if ($toKunena)
        {
            if (!empty($websitename))
            {
                $mapped['websitename'] = $this->_userdata[$websitename];
            }
            $mapped[$to] = substr($this->_userdata[$from], strpos($this->_userdata[$from], '://') + 3);
        }
        else
        {
            if (!empty($websitename) && $websitename != $to)
            {
                $mapped[$websitename] = $this->_userdata['websitename']; 
            }
            $mapped[$to] = 'http://' . $this->_userdata[$from];
        }
    }

    protected function _messageorderingMapper(&$mapped, $from, $to, $toKunena)
    {
        $asc  = $this->params->get('map_ordering_asc', 'Asc');
        $desc = $this->params->get('map_ordering_desc', 'Desc');

        $map  = array(0 => $asc, 1 => $desc);

        $mapped[$to] = $this->_mapArrayHelper($map, $from, $toKunena);
    }

    protected function _hidemailMapper(&$mapped, $from, $to, $toKunena)
    {
        $no  = $this->params->get('map_no', 'No');
        $yes = $this->params->get('map_yes', 'Yes');

        $map  = array(0 => $no, 1 => $yes);

        $mapped[$to] = $this->_mapArrayHelper($map, $from, $toKunena);
    }

    protected function _showonlineMapper(&$mapped, $from, $to, $toKunena)
    {
        $this->_hidemailMapper($mapped, $from, $to, $toKunena);
    }
}
